import { Request, Response } from "express";
import CheckIn from "../db/models/CheckIn";
import Boat from "../db/models/Boat";
import BoatType from "../db/models/BoatType";

//create log entry
const createLogEntryController = async (req: Request, res: Response) => {
  try {

    //get input
    const logEntryInput = req.body;

    //find related boat using given boatId
    const chosenBoat = await Boat.findByPk(logEntryInput.boatId);

    //if boat not found
    if (!chosenBoat) {
      return res
        .status(404)
        .json({ success: false, error: "givenBoatIdNotFound" });
    }

    //get status of boat
    const bootAvailability = chosenBoat.status;

    //if boat status is false => is not available => boat locked
    if (bootAvailability === false) {
      return res.status(409).json({ success: false, error: "BoatLocked" });
    }

    //check if number of additional clientsNames == given num of all clients - 1 (without responsible client)
    const numOfAdditionalClients = logEntryInput.additionalClients.length;
    const numP = logEntryInput.numP;
    
    if (numOfAdditionalClients !== numP-1) { //-1 because of the responsible Person
      return res.status(404).json({
        success: false,
        error: "numberOfAllGivenClientsNamesDoesNotMatchNumberOfAllClients",
      });
    }
    
    //check of numOfAllClients >= minP && numOfAllClients <= maxP
    const minP = chosenBoat.minP;
    const maxP = chosenBoat.maxP;

    if (logEntryInput.numP >= minP && logEntryInput.numP <= maxP) { 
      return res.status(404).json({
        success: false,
        error: "numberOfAllGivenClientsDoesNotMatchRequirementsOfChosenBoat",
      });
    }



    //create entry
    const newLogEntry = await CheckIn.create(logEntryInput);

    //return result after checking all possible error-cases
    return res.status(201).json({
      success: true,
      result: {
        startTime: newLogEntry.startTime,
        estimatedEndTime: newLogEntry.estimatedEndTime,
        email: newLogEntry.email,
        fullNameOfResponsableClient: newLogEntry.fullNameOfResponsibleClient,
        additionalClients: newLogEntry.additionalClients,
        boatID: newLogEntry.boatId,
      },
    });
  } catch (error) {
    console.error(error.message);
    return res.status(500).json({ success: false, error: "serverError" });
  }
};
export default createLogEntryController;
