import { DataTypes, Model, Optional, Sequelize } from "sequelize";

interface CheckInAttributes {
  id: string;
  date: Date;
  startTime: Date;
  estimatedEndTime: Date;
  destination: string;
  email: string;
  fullNameOfResponsibleClient: string;
  additionalClients: Array<string>;
  boatId: string;
  numP: number;
  returned: boolean;
  note: string;
  bookingType: string;
}
export interface CheckInAttributesInput
  extends Optional<CheckInAttributes, "id"> {}

class CheckIn
  extends Model<CheckInAttributes, CheckInAttributesInput>
  implements CheckInAttributes
{
  declare id: string;
  declare date: Date;
  declare destination: string;
  declare startTime: Date;
  declare estimatedEndTime: Date;
  declare email: string;
  declare fullNameOfResponsibleClient: string;
  declare additionalClients: Array<string>;
  declare boatId: string;
  declare numP: number;
  declare returned: boolean;
  declare note: string;
  declare bookingType: string;

  declare readonly createdAt: Date;
  declare readonly updatedAt: Date;
}

export const initCheckIn = async (sequelizeConnection: Sequelize) => {
  CheckIn.init(
    {
      id: {
        type: DataTypes.UUID,
        defaultValue: DataTypes.UUIDV4,
        primaryKey: true,
        allowNull: false,
      },
      numP: {
        type: DataTypes.INTEGER,
        allowNull: false,
      },
      date: {
        type: DataTypes.DATE,
        allowNull: false,
      },
      destination: {
        type: DataTypes.STRING(),
        allowNull: true,
      },
      startTime: {
        type: DataTypes.DATE,
        allowNull: false,
      },
      estimatedEndTime: {
        type: DataTypes.DATE,
        allowNull: false,
      },
      email: {
        type: new DataTypes.STRING(),
        allowNull: false,
      },
      fullNameOfResponsibleClient: {
        type: new DataTypes.STRING(),
        allowNull: false,
      },
      additionalClients: {
        type: new DataTypes.ARRAY(DataTypes.STRING),
      },
      boatId: {
        type: DataTypes.UUID,
        references: {
          model: "boat",
          key: "id",
        },
      },
      bookingType: {
        type: DataTypes.STRING(),
        allowNull: false,
      },
      returned: {
        type: DataTypes.BOOLEAN,
        allowNull: false,
      },
      note: {
        type: DataTypes.STRING(),
        allowNull: true,
      },
    },
    {
      tableName: "checkin",
      sequelize: sequelizeConnection,
    }
  );
};

export default CheckIn;
